(import
 (owl toplevel)
 (owl regex)
 (owl thread)
 (prefix (owl sys) sys/)
 (prefix (sqlite) s3/))

(define (P s) (write-bytes stdout (string->list s)))
(define (caar* v) (car* (car* v)))
(define (but-last* l)
  (let loop ((l l) (acc ()))
    (if (null? (cdr* l))
        (values acc (car* l))
        (loop (cdr l) (append acc (list (car l)))))))

(define (last-id ptr)
  (caar* (s3/execute* ptr "select tid from links order by id desc limit 1" ())))

(define (new-id ptr)
  (let ((lid (string->list (last-id ptr))))
    (if (> (+ (last lid 0) 1) #\z)
        (list->string (make-list (+ (length lid) 1) #\a))
        (lets ((l r (but-last* lid)))
          (list->string (append l (list (+ r 1))))))))

(define (add-link ptr l)
  (let ((id (new-id ptr)))
    (s3/execute* ptr "insert into links (tid, target) values (?,?)" (list id l))
    id))

(define (get-link ptr tid)
  (caar* (s3/execute* ptr "select target from links where tid = ?" (list tid))))

(define qsplit (string->regex "c/\\?/"))

(define (lref* l n)
  (try (λ () (lref l n)) ""))

(λ (args)
  (let* ((ptr (s3/open "db.sqlite"))
         (uri (sys/getenv "REQUEST_URI"))
         (l (qsplit (if uri uri ""))))
    (s3/execute ptr "create table if not exists links (id integer primary key, tid text, target text)")

    (cond
     ((string=? (lref* args 1) "add") ;; ran from command line, append next arg to db and print tid
      (print (add-link ptr (lref args 2)))
      0)
     ((> (len l) 1)
      (let ((l (get-link ptr (last l "a"))))
        (if (null? l)
            (P "Content-type: text/plain\r\n\r\ninvalid request: no such link")
            (P (string-append "Location: " l "\r\n\r\n")))))
     (else
      (P "Content-type: text/plain\r\n\r\ninvalid request")))

    (s3/close ptr))
  0)
